using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;

using NETCore.MailKit.Extensions;
using NETCore.MailKit.Infrastructure.Internal;

using TestProject.Data;
using TestProject.Models;

namespace TestProject
{
    /// <summary>
    /// 
    /// </summary>
    public class Startup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 
        /// </summary>
        private IConfiguration configuration;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Startup(configuration)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="configuration"></param>
        public Startup(IConfiguration configuration)
        {
            this.configuration = configuration;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  ÷ ϱ - ConfigureServices(services)

        /// <summary>
        ///  ÷ ϱ
        /// </summary>
        /// <param name="services"> ÷</param>
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddDbContext<DatabaseContext>(options => { options.UseInMemoryDatabase("TestDB"); });

            services.AddIdentity<ApplicationUser, IdentityRole>
            (
                options =>
                {
                    options.Password.RequiredLength         = 4;
                    options.Password.RequireDigit           = false;
                    options.Password.RequireNonAlphanumeric = false;
                    options.Password.RequireUppercase       = false;
                    options.SignIn.RequireConfirmedEmail    = true;
                }
            )
            .AddEntityFrameworkStores<DatabaseContext>()
            .AddDefaultTokenProviders();

            services.ConfigureApplicationCookie
            (
                options =>
                {
                    options.Cookie.Name = "TestCookie";
                    options.LoginPath   = "/Home/Login";
                }
            );

            services.AddMailKit
            (
                options =>
                {
                    options.UseMailKit(this.configuration.GetSection("MailAddress").Get<MailKitOptions>());
                }
            );

            services.AddControllersWithViews();
        }

        #endregion
        #region ϱ - Configure(app, environment)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="app">ø̼ </param>
        /// <param name="environment"> ȣƮ ȯ</param>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment environment)
        {
            if(environment.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseAuthentication();

            app.UseAuthorization();

            app.UseEndpoints
            (
                endpoints =>
                {
                    endpoints.MapDefaultControllerRoute();
                }
            );
        }

        #endregion
    }
}